@@ -8,6 +8,7 @@
 """
 """
 import flexsolve as flx
+from numba import njit
 from warnings import warn
 from ..exceptions import InfeasibleRegion
 from . import binary_phase_fraction as binary
@@ -15,12 +16,29 @@ from .equilibrium import Equilibrium
 from .dew_point import DewPointCache
 from .bubble_point import BubblePointCache
 from .fugacity_coefficients import IdealFugacityCoefficients
+from .poyinting_correction_factors import IdealPoyintingCorrectionFactors
+from . import activity_coefficients as ac
 from .. import functional as fn
 from ..utils import Cache
 import numpy as np
 
 __all__ = ('VLE', 'VLECache')
 
+# @njit#(cache=True)
+def xV_iter(xV, Psat_over_P_phi, T, z, z_light, z_heavy, f_gamma, gamma_args, f_pcf, pcf_args):
+    xV = xV.copy()
+    x = xV[:-1]
+    V = xV[-1]
+    x[x < 0.] = 0.
+    x = fn.normalize(x)
+    Ks = Psat_over_P_phi * f_gamma(x, T, *gamma_args) * f_pcf(x, T, *pcf_args)
+    xV[-1] = V = binary.phase_fraction(z, Ks, V,
+                                       z_light,
+                                       z_heavy)
+    xV[:-1] = z/(1. + V * (Ks - 1.))
+    return xV
+    
+
 class VLE(Equilibrium, phases='lg'):
     """
     Create a VLE object that performs vapor-liquid equilibrium when called.
@@ -172,7 +190,9 @@ class VLE(Equilibrium, phases='lg'):
                  '_V', # [float] Molar vapor fraction.
                  '_dew_point', # [DewPoint] Solves for dew point.
                  '_bubble_point', # [BubblePoint] Solves for bubble point.
-                 '_x', # [1d array] Liquid composition.
+                 '_x', # [1d array] Vapor composition.
+                 '_y', # [1d array] Liquid composition.
+                 '_z_last', # tuple[1d array] Last bulk composition.
                  '_phi', # [FugacityCoefficients] Estimates fugacity coefficients of gas.
                  '_pcf', # [PoyintingCorrectionFactors] Estimates the PCF of a liquid.
                  '_gamma', # [ActivityCoefficients] Estimates activity coefficients of a liquid.
@@ -189,7 +209,6 @@ class VLE(Equilibrium, phases='lg'):
                  '_z_norm', # [1d array] Normalized molar composition of chemicals in equilibrium
                  '_z_light', # [1d array] Molar composition of light chemicals not included in equilibrium calculation.
                  '_z_heavy', # [1d array] Molar composition of heavy chemicals not included in equilibrium calculation.
-                 '_Ks', # [1d array] Partition coefficients.
                  '_nonzero', # [1d array(bool)] Chemicals present in the mixture
                  '_F_mol', # [float] Total moles of chemicals (accounting for dissociation).
                  '_F_mol_vle', # [float] Total moles in equilibrium.
@@ -209,6 +228,8 @@ class VLE(Equilibrium, phases='lg'):
         self._bubble_point_cache = bubble_point_cache or BubblePointCache()
         super().__init__(imol, thermal_condition, thermo)
         imol = self._imol
+        self._x = None
+        self._z_last = None
         self._phase_data = tuple(imol)
         self._liquid_mol = liquid_mol = imol['l']
         self._vapor_mol = imol['g']
@@ -509,12 +530,7 @@ class VLE(Equilibrium, phases='lg'):
         dP = (P_bubble - P_dew)
         V = (P - P_dew) / dP if dP > 1. else 0.5
         self._refresh_v(V, y_bubble)
-        # Solve
-        try:
-            v = self._solve_v(T, P)
-        except:
-            self._v = self._estimate_v(V, y_bubble)
-            v = self._solve_v(T, P)
+        v = self._solve_v(T, P)
         self._vapor_mol[self._index] = v
         self._liquid_mol[self._index] = self._mol_vle - v
         self._H_hat = self.mixture.xH(self._phase_data, T, P)/self._F_mass
@@ -740,19 +756,25 @@ class VLE(Equilibrium, phases='lg'):
                                      H_hat_bubble - H_hat, H_hat_dew - H_hat,
                                      self._T, self.T_tol, self.H_hat_tol,
                                      (H_hat,), checkiter=False, checkbounds=False)
-        
+            # self._T = thermal_condition.T = T
             # Make sure enthalpy balance is correct
             self._T = thermal_condition.T = self.mixture.xsolve_T(
                 self._phase_data, H, T, P
             )
-            self._H_hat = H_hat
+        self._H_hat = H_hat
     
     def _estimate_v(self, V, y_bubble):
         return (V*self._z_norm + (1-V)*y_bubble) * V * self._F_mol_vle
     
     def _refresh_v(self, V, y_bubble):
-        self._v = self._estimate_v(V, y_bubble)
+        self._v = v = self._estimate_v(V, y_bubble)
         self._V = V
+        self._y = fn.normalize(v, v.sum() + self._F_mol_light)
+        z_last = self._z_last
+        if (self._x is None
+            or np.abs(z_last - self._z).sum() > 0.001):
+            l = self._mol_vle - v
+            self._x = fn.normalize(l, l.sum() + self._F_mol_heavy)
     
     def _H_hat_err_at_T(self, T, H_hat):
         self._vapor_mol[self._index] = self._solve_v(T, self._P)
@@ -772,24 +794,31 @@ class VLE(Equilibrium, phases='lg'):
     def _V_err_at_T(self, T, V):
         return self._solve_v(T, self._P).sum()/self._F_mol_vle  - V
     
-    def _x_iter(self, x, Psat_over_P_phi):
-        x = fn.normalize(x)
-        x[x < 1e-32] = 1e-32
-        self._Ks = Psat_over_P_phi * self._gamma(x, self._T) * self._pcf(x, self._T)
-        self._V = V = binary.phase_fraction(self._z, self._Ks, self._V,
-                                            self._z_light,
-                                            self._z_heavy)
-        return self._z/(1. + V * (self._Ks - 1.))
-    
     def _y_iter(self, y, Psats_over_P, T, P):
         phi = self._phi(y, T, P)
+        gamma = self._gamma
+        pcf = self._pcf
+        x = self._x
         Psat_over_P_phi = Psats_over_P / phi
-        f = self._x_iter
-        args = (Psat_over_P_phi,)
-        x = flx.aitken(f, self._x, 1e-12, args, checkiter=False, 
+        f = xV_iter
+        z = self._z
+        args = (Psat_over_P_phi, T, z, self._z_light, 
+                self._z_heavy, gamma.f, gamma.args, pcf.f, pcf.args)
+        xV = np.zeros(x.size + 1)
+        xV[:-1] = x
+        xV[-1] = self._V
+        xV = flx.aitken(f, xV, 1e-12, args, checkiter=False, 
                        checkconvergence=False, convergenceiter=3)
-        self._x = f(x, *args)
-        v = self._F_mol * self._V * x * self._Ks     
+        x = xV[:-1]
+        self._V = V = xV[-1]
+        x[x < 1e-32] = 1e-32
+        self._x = xV[:-1] = x = fn.normalize(x)
+        if V == 0:
+            Ks = 0
+        else:
+            Ks = (z / x - 1) / V + 1.
+        self._z_last = z
+        v = self._F_mol * V * x * Ks     
         return fn.normalize(v, v.sum() + self._F_mol_light)
     
     def _solve_v(self, T, P):
@@ -797,14 +826,10 @@ class VLE(Equilibrium, phases='lg'):
         Psats_over_P = np.array([i(T) for i in
                                  self._bubble_point.Psats]) / P
         self._T = T
-        v = self._v
-        y = fn.normalize(v, v.sum() + self._F_mol_light)
-        l = self._mol_vle - v
-        self._x = fn.normalize(l, l.sum() + self._F_mol_heavy)
         if isinstance(self._phi, IdealFugacityCoefficients):
-            y = self._y_iter(y, Psats_over_P, T, P)
+            y = self._y_iter(self._y, Psats_over_P, T, P)
         else:
-            y = flx.aitken(self._y_iter, v/v.sum(), 1e-12,
+            y = flx.aitken(self._y_iter, self._y, 1e-12,
                            args=(Psats_over_P, T, P),
                            checkiter=False, 
                            checkconvergence=False, 
